#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const util_1 = require("util");
const rimraf = require("rimraf");
const delay = require("delay");
const globby = require("globby");
const prettyMs = require("pretty-ms");
const minimist = require("minimist");
const imagemin = require("imagemin");
const imageminJpegtran = require("imagemin-jpegtran");
const imageminPngquant = require("imagemin-pngquant");
const config_1 = require("../lib/config");
const write_api_data_1 = require("../lib/write-api-data");
const util_2 = require("../lib/util");
const CWD = process.cwd();
const { version } = (() => {
    const fileName = path.join(__dirname, '..', 'package.json');
    try {
        return JSON.parse(fs_1.readFileSync(fileName, 'utf8'));
    }
    catch (_a) {
        return { version: 'unknown' };
    }
})();
const argv = minimist(process.argv.slice(2), {
    boolean: ['h', 'help', 'V', 'version'],
    alias: {
        h: 'help',
        V: 'version',
    },
});
if (argv.help) {
    process.stdout.write(`
🦊  Build & export your awesome photo site! 🦊

Usage
  $ npx phox

Options
  --help, -h     Show this Help section
  --version, -V  Displays the phox version you're currently using

To configure the build, add a phox.config.js to the root of your
project and put the desired options into it.

Version: ${version}
`);
    process.exit();
}
if (argv.version) {
    util_2.log(`v${version}`);
    process.exit();
}
const config = config_1.default();
const outDir = path.join(CWD, config.outDir);
const pExec = util_1.promisify(child_process_1.exec);
const pStat = util_1.promisify(fs_1.stat);
const now = Date.now();
const NO_SERVER_JS_ERR = `
./${config.server} is missing.

Visit https://github.com/herschel666/phox#setup
to learn how to set up the local dev-server.
`;
const minifyImages = () => __awaiter(this, void 0, void 0, function* () {
    const plugins = [imageminJpegtran(), imageminPngquant({ quality: '65-80' })];
    const folder = path.join(outDir, 'static', config.albumsDir);
    const albums = yield globby(path.join(folder, '*'));
    yield Promise.all(albums.map((album) => __awaiter(this, void 0, void 0, function* () {
        return imagemin([path.join(album, '*.{jpg,png}')], album, {
            plugins,
        });
    })));
});
const serverJsExists = () => __awaiter(this, void 0, void 0, function* () {
    try {
        yield pStat(config.server);
        return true;
    }
    catch (_a) {
        return false;
    }
});
process.env.NODE_ENV = 'production';
const main = () => __awaiter(this, void 0, void 0, function* () {
    let server = null;
    try {
        const hasServerJs = yield serverJsExists();
        if (!hasServerJs) {
            return {
                error: { message: NO_SERVER_JS_ERR },
            };
        }
        util_2.log('Cleaning up ...');
        rimraf.sync(outDir);
        util_2.log('Building the site ...');
        yield pExec('npx next build');
        util_2.log('Starting the dev-server ...');
        server = child_process_1.spawn('node', [config.server]);
        yield delay(1000);
        util_2.log('Exporting the site ...');
        yield pExec(`npx next export -o ${outDir}`);
        yield write_api_data_1.default(config);
        util_2.log('Minifying the images ...');
        yield minifyImages();
    }
    catch (error) {
        return { server, error };
    }
    return { server };
});
const checkResult = ({ server, error }) => {
    const exitCode = error ? 1 : 0;
    if (error) {
        util_2.log('Something went wrong while phoxing ...');
        console.error('  ', error.message);
        if (error.stack) {
            // tslint:disable-next-line:no-console
            console.log('  ', error.stack);
        }
    }
    else {
        const ms = Date.now() - now;
        const duration = prettyMs(ms);
        util_2.log(`Successfully built your site in ${duration}.`);
    }
    return { server, exitCode };
};
const killServer = ({ server, exitCode }) => __awaiter(this, void 0, void 0, function* () {
    if (server) {
        process.kill(server.pid);
        yield new Promise(innerResolve => server.on('close', innerResolve));
    }
    process.exit(exitCode);
});
main()
    .then(checkResult)
    .then(killServer)
    .catch(console.error);
